﻿using System;
using System.Data;
using System.IO;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace GridDragDropScrollCell
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            BaseGridBehavior gridBehavior = this.radGridView1.GridBehavior as BaseGridBehavior;
            BaseGridBehavior gridBehavior2 = this.radGridView2.GridBehavior as BaseGridBehavior;

            this.RegisterBehavior(gridBehavior);
            this.RegisterBehavior(gridBehavior2);

            this.radGridView1.CellFormatting += this.OnCellFormatting;
            this.radGridView2.CellFormatting += this.OnCellFormatting;

            this.SubscribeService(this.radGridView1.GridViewElement);
            this.SubscribeService(this.radGridView2.GridViewElement);
        }

        private void SubscribeService(RadGridViewElement gridViewElement)
        {
            RadGridViewDragDropService service = gridViewElement.GetService<RadGridViewDragDropService>();
            service.PreviewDragOver += this.OnPreviewDragOver;
            service.PreviewDragDrop += this.OnPreviewDragDrop;
        }

        private void OnPreviewDragDrop(object sender, Telerik.WinControls.RadDropEventArgs e)
        {
            GridDataCellElement dataCellElement = e.HitTarget as GridDataCellElement;
            CellSnapshotDragItem item = e.DragInstance as CellSnapshotDragItem;

            if (item != null && dataCellElement != null)
            {
                dataCellElement.Value = ((item as ISupportDrag).GetDataContext() as GridViewCellInfo).Value;
                e.Handled = true;
            }
        }

        private void OnPreviewDragOver(object sender, Telerik.WinControls.RadDragOverEventArgs e)
        {
            if (e.HitTarget is GridDataCellElement)
            {
                e.CanDrop = true;
            }
        }

        private void RegisterBehavior(BaseGridBehavior gridBehavior)
        {
            gridBehavior.UnregisterBehavior(typeof(GridViewDataRowInfo));
            gridBehavior.RegisterBehavior(typeof(GridViewDataRowInfo), new CustomGridDataRowBehavior());
        }

        private void OnCellFormatting(object sender, CellFormattingEventArgs e)
        {
            e.CellElement.AllowDrag = true;
            e.CellElement.AllowDrop = true;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.radGridView1.DataSource = this.GetDataSource();
            this.radGridView2.DataSource = this.GetDataSource();
        }

        private DataTable GetDataSource()
        {
            DataTable table = new DataTable("table");

            int columns = 100;
            int rows = 100;

            for (int i = 0; i < columns; i++)
            {
                table.Columns.Add("Column" + i);
            }

            for (int i = 0; i < rows; i++)
            {
                object[] values = new object[columns];

                for (int j = 0; j < columns; j++)
                {
                    values[j] = Path.GetRandomFileName();
                }

                table.Rows.Add(values);
            }

            return table;
        }
    }
}
